#include "account_media.hpp"
#include <ts/generated.h>
#include <iostream>
#include <tracer/tracer.hpp>

using namespace cpptesk::tracer::coverage;

void trace_deposit_coverage_structure()
{
    static int traceId = -1;
    if (traceId>= ::getTraceId()) return;
    traceId = ::getTraceId();
    ::traceCoverageStructureStart("");
    cpptesk::tracer::TracerEndFunction _traceModelOp(::traceCoverageStructureEnd);

    traceCoverageStructure<AccountModel::DepositStart>();
    traceCoverageStructure<AccountModel::DepositEnd>();
    traceCoverageStructure<AccountModel::Deposit>();
}
void trace_withdraw_coverage_structure()
{
    static int traceId = -1;
    if (traceId>= ::getTraceId()) return;
    traceId = ::getTraceId();
    ::traceCoverageStructureStart("");
    cpptesk::tracer::TracerEndFunction _traceModelOp(::traceCoverageStructureEnd);

    traceCoverageStructure<AccountModel::WithdrawStart>();
    traceCoverageStructure<AccountModel::WithdrawEnd>();
    traceCoverageStructure<AccountModel::Withdraw>();
}


bool AccountMediator::deposit(int sum)
{
    ::traceModelOperationStart("stimulus", "", "int Account::deposit(int)");
    cpptesk::tracer::TracerEndFunction _traceModelOp(::traceModelOperationEnd);
    ::traceOracleStart("", "int Account::deposit(int)", 1);
    cpptesk::tracer::TracerEndFunction _traceOracle(::traceOracleEnd);

    trace_deposit_coverage_structure();

    if (!pre_deposit(sum))
    {
        std::cout << "AccountMediator::deposit::pre condition failed" << std::endl;
        ::traceException(PRECONDITION_FAILED_MESSAGE);
        return false;
    }
    ::tracePreconditionEnd();

    cov_deposit(sum);

    int res = impl->deposit(sum);
    int pre_balance = balance;
    balance += res;
    bool result = post_deposit(pre_balance, sum, res);
    if (!result)
    {
        std::cout << "AccountMediator::deposit::post condition failed" << std::endl;
        ::traceException(POSTCONDITION_FAILED_MESSAGE);
    }

    return result;
}

bool AccountMediator::withdraw(int sum)
{
    ::traceModelOperationStart("stimulus", "", "int Account::withdraw(int)");
    cpptesk::tracer::TracerEndFunction _traceModelOp(::traceModelOperationEnd);
    ::traceOracleStart("", "int Account::withdraw(int)", 2);
    cpptesk::tracer::TracerEndFunction _traceOracle(::traceOracleEnd);

    trace_withdraw_coverage_structure();

    if (!pre_withdraw(sum))
    {
        std::cout << "AccountMediator::withdraw::pre condition failed" << std::endl;
        ::traceException(PRECONDITION_FAILED_MESSAGE);
        return false;
    }

    ::tracePreconditionEnd();

	cov_withdraw(sum);

    int res = impl->withdraw(sum);

    int pre_balance = balance;
    balance -= res;
    bool result = post_withdraw(pre_balance, sum, res);
    if (!result)
    {
        std::cout << "AccountMediator::withdraw::post condition failed" << std::endl;
        ::traceException(POSTCONDITION_FAILED_MESSAGE);
    }
    return result;
}

bool AccountMediator::get_balance()
{
    int pre_balance = balance;
    int result = impl->get_balance();
    balance = result;
    return post_get_balance(pre_balance, result);
}

AccountMediator::AccountMediator() : AccountModel(), impl(0) {}
AccountMediator::AccountMediator(Account* impl) : AccountModel(impl->get_limit()), impl(impl) {}
AccountMediator::~AccountMediator(void) {}

