/** @file counter_model.cpp
 * This file contains implementation of the counter model class.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */


#include <counter_model.h>

namespace cpptesk {
namespace examples {
namespace counter {

Counter::Counter() : coverage(*this)
{
    ADD_INPUT(iface1);
    ADD_COMPONENT(iface2);
}

Counter::~Counter() {}

bool Counter::isIface1Ready() const
{
    return iface1.isReady();
}

DEFINE_STIMULUS(Counter::inc)
{
    InputData data = CAST_MESSAGE(InputData);

    START_STIMULUS(PARALLEL);

    OutputData odata = OutputData(get_value());

    inc_counter();

    CYCLE();

    read_result(process, iface2, odata);

    STOP_STIMULUS();
}

DEFINE_REACTION(Counter::read_result)
{
    START_PROCESS();
    SEND_REACTION(PARALLEL, GET_IFACE(), GET_MESSAGE());
    STOP_PROCESS();
}

void Counter::inc_counter(void)
{
    counter = (counter + 1) % 8;
}

int Counter::get_value(void)
{
    return counter;
}

void Counter::refresh_coverage(void)
{
    coverage.out_cov();
}

}}}

