#include "account_scen.h"
#include <tracer/tracer.hpp>

using namespace cpptesk::ts;

bool AccountScenario::init(int argc, char ** argv)
{
    return true;
}

void AccountScenario::finish() {}

bool AccountScenario::deposit_scenario(IntCtx& ctx)
{
    IBEGIN
    for (IVAR(i) = 1; IVAR(i) + mediator.balance < limit; ++ IVAR(i))
    {
        IACTION
        {
            // std::cout << "AccountScenario::deposit(" << IVAR(i) << ')' << std::endl;
            YIELD (mediator.deposit(IVAR(i)));
        }
    }
    IEND
}

bool AccountScenario::withdraw_scenario(IntCtx& ctx)
{
    IBEGIN
    for (IVAR(i) = 1; mediator.balance - IVAR(i) > -limit; ++ IVAR(i))
    {
        IACTION
        {
            // std::cout << "AccountScenario::withdraw(" << IVAR(i) << ')' << std::endl;
            YIELD (mediator.withdraw(IVAR(i)));
        }
    }
    IEND
}

#ifdef NDFSM
bool AccountScenario::random_withdraw_scenario(IntCtx& ctx)
{
    IBEGIN
    IACTION
    {
        // std::cout << "AccountScenario::random_withdraw()" << std::endl;
        YIELD(mediator.withdraw(rand() % limit));
    }
    IEND
}
#endif

bool AccountScenario::get_state_scen()
{
    // std::cout << "AccountScenario::get_state_scen " << std::endl;
    return mediator.get_balance();
}

int AccountScenario::get_state_int()
{
    // std::cout << "Current state: " << mediator.balance << std::endl;
    return mediator.balance;
}

AccountScenario::AccountScenario(int limit, Account* impl) : ScenarioBase(), mediator(impl), limit(limit)
{
    setup("Account Test"
          , &AccountScenario::init, &AccountScenario::finish, &AccountScenario::get_state_int
         );
    CPPTESK_ADD_SCENARIO_METHOD(AccountScenario::deposit_scenario);
    CPPTESK_ADD_SCENARIO_METHOD(AccountScenario::withdraw_scenario);
#ifdef NDFSM
    CPPTESK_ADD_SCENARIO_METHOD(AccountScenario::random_withdraw_scenario);
#endif
}
AccountScenario::AccountScenario(void) : ScenarioBase()
{
    setup("Account Test"
          , &AccountScenario::init, &AccountScenario::finish, &AccountScenario::get_state_int
         );
    CPPTESK_ADD_SCENARIO_METHOD(AccountScenario::deposit_scenario);
    CPPTESK_ADD_SCENARIO_METHOD(AccountScenario::withdraw_scenario);
#ifdef NDFSM
    CPPTESK_ADD_SCENARIO_METHOD(AccountScenario::random_withdraw_scenario);
#endif
}
AccountScenario::~AccountScenario(void) {}

