/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <assert.h>
#include <utils/parparser.h>
#include <utils/testreg.h>

static CppTestScenario *local_initTestScenario_cpp;
static CppTestScenario *local_testScenario_cpp;
static TestEngine *local_testEngine_cpp;
static int local_argc_cpp;
static char **local_argv_cpp;
static int local_netscen_cpp;

extern "C" int start_scenario_with_wrapper(void(*wrapper)(void));

void set_scenario_wrapper(CppTestScenario *testScenario, TestEngine *testEngine, char *parameters, bool netscen)
{
    local_testScenario_cpp = testScenario;
    local_testEngine_cpp = testEngine;
    local_netscen_cpp = netscen;
    local_argc_cpp = parse_parameters(parameters, local_testScenario_cpp->name, &local_argv_cpp);
}

void set_scenario_wrapper_with_init(CppTestScenario *initTestScenario, CppTestScenario *testScenario, TestEngine *testEngine, char *parameters, bool netscen)
{
    local_initTestScenario_cpp = initTestScenario;
    set_scenario_wrapper(testScenario, testEngine, parameters, netscen);
}

void scenario_wrapper_cpp(void)
{
    if(!local_netscen_cpp)
        localmain(local_testEngine_cpp, local_testScenario_cpp, local_argc_cpp, local_argv_cpp);
    else
        netmain(local_testEngine_cpp, local_testScenario_cpp, local_argc_cpp, local_argv_cpp);
}

void scenario_wrapper_with_init_cpp(void)
{
    localmain(local_testEngine_cpp, local_initTestScenario_cpp, local_argc_cpp, local_argv_cpp);
    scenario_wrapper_cpp();
}

int startscen_cpp_TestRegistry(bool netscen, char *scen_name, char *scen_params)
{
    int scen_number = find_scenario_TestRegistry(scen_name, scen_params);

    if(test_registry[scen_number].testType == CTESK) {
        TestScenario* ctest = (TestScenario*)test_registry[scen_number].testScenario;
        set_scenario_wrapper(ctest, test_registry[scen_number].testEngine, scen_params, netscen);
        return start_scenario_with_wrapper(scenario_wrapper);
    }
    else if(test_registry[scen_number].testType == CPPTESK) {
        CppTestScenario* cpptest = ((cpptesk::ts::ScenarioBase*)test_registry[scen_number].testScenario)->getTestScenario();
        set_scenario_wrapper(cpptest, test_registry[scen_number].testEngine, scen_params, netscen);
        return start_scenario_with_wrapper(scenario_wrapper_cpp);
    }
    else {
        assert(false);
        return 1;
    }
}

/*
int start_scenario(CppTestScenario *scen, char *params)
{
    set_scenario_wrapper(scen, &TE_NDFSM, params, false);
    return start_scenario_with_wrapper(scenario_wrapper_cpp);
}

int start_net_scenario(CppTestScenario *scen, char *params)
{
    set_scenario_wrapper(scen, &TE_NDFSM, params, true);
    return start_scenario_with_wrapper(scenario_wrapper_cpp);
}

int start_scenario_with_init(CppTestScenario *initscen, cpptesk::ts::CppTestScenario *scen, char *params)
{
    set_scenario_wrapper_with_init(initscen, scen, &TE_NDFSM, params, false);
    return start_scenario_with_wrapper(scenario_wrapper_with_init_cpp);
}

int start_net_scenario_with_init(CppTestScenario *initscen, cpptesk::ts::CppTestScenario *scen, char *params)
{
    set_scenario_wrapper_with_init(initscen, scen, &TE_NDFSM, params, true);
    return start_scenario_with_wrapper(scenario_wrapper_with_init_cpp);
}

int start_scenario_new(CppTestScenario *scen, TestEngine *testEngine, char *params, bool netscen)
{
    set_scenario_wrapper(scen, testEngine, params, netscen);
    return start_scenario_with_wrapper(scenario_wrapper_cpp);
}

int start_scenario_with_init_new(CppTestScenario *initscen, CppTestScenario *scen, TestEngine *testEngine, char *params, bool netscen)
{
    set_scenario_wrapper_with_init(initscen, scen, testEngine, params, netscen);
    return start_scenario_with_wrapper(scenario_wrapper_with_init_cpp);
}
*/

